@@ -822,7 +822,9 @@ char* process_metachars(const char* input)
 				break;
 		}
 	}
-	copy[j] = '\0';
+	copy[i] = '\0';
+
+	return copy;
 }
 
 /* read in the configuration file */
@@ -1912,9 +1914,9 @@ void handle_connection(int sock)
 
 	} else {
 
-		pkt_size = (sizeof(v3_packet) - NRPE_V4_PACKET_SIZE_OFFSET) + strlen(send_buff);
+		pkt_size = (sizeof(v3_packet) - NRPE_V4_PACKET_SIZE_OFFSET) + strlen(send_buff) + 1;
 		if (packet_ver == NRPE_PACKET_VERSION_3) {
-			pkt_size = (sizeof(v3_packet) - NRPE_V3_PACKET_SIZE_OFFSET) + strlen(send_buff);
+			pkt_size = (sizeof(v3_packet) - NRPE_V3_PACKET_SIZE_OFFSET) + strlen(send_buff) + 1;
 		}
 		v3_send_packet = calloc(1, pkt_size);
 		send_pkt = (char *)v3_send_packet;
@@ -1923,7 +1925,7 @@ void handle_connection(int sock)
 		v3_send_packet->packet_type = htons(RESPONSE_PACKET);
 		v3_send_packet->result_code = htons(result);
 		v3_send_packet->alignment = 0;
-		v3_send_packet->buffer_length = htonl(strlen(send_buff));
+		v3_send_packet->buffer_length = htonl(strlen(send_buff) + 1);
 		strcpy(&v3_send_packet->buffer[0], send_buff);
 
 		/* calculate the crc 32 value of the packet */
@@ -2748,10 +2750,6 @@ int validate_request(v2_packet * v2pkt, v3_packet * v3pkt)
 	if (packet_ver >= NRPE_PACKET_VERSION_3) {
 
 		buffer_size = ntohl(v3pkt->buffer_length);
-		if (buffer_size < 0 || buffer_size > INT_MAX - pkt_size) {
-			logit(LOG_ERR, "Error: Request packet had invalid buffer size.");
-			return ERROR;
-		}
 
 		pkt_size = sizeof(v3_packet);
 		pkt_size -= (packet_ver == NRPE_PACKET_VERSION_3 ? NRPE_V3_PACKET_SIZE_OFFSET : NRPE_V4_PACKET_SIZE_OFFSET);
